package var.mom.jms.chat;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * Chat Client using publish/subscribe on JMS provider Topic; represents user
 * name as String property of message
 *
 * @author Sandro Leuchter
 *
 */
public class ChatClientUserProperty implements MessageListener {
	private Connection connection;
	private Session session;
	private MessageProducer producer;
	private MessageConsumer consumer;

	/**
	 * constructor, establishes and starts connection to JMS provider specified in
	 * JNDI (via jndi.properties), afterwards producer and consumer are ready
	 *
	 * @param sendDest    Destination for producer
	 * @param receiveDest Destination for consumer
	 *
	 * @throws NamingException JNDI exceptions
	 * @throws JMSException    JMS exceptions
	 * @see javax.jms.Destination
	 */
	public ChatClientUserProperty(String sendDest, String receiveDest) throws NamingException, JMSException {
		Context ctx = new InitialContext();
		ConnectionFactory factory = (ConnectionFactory) ctx.lookup("ConnectionFactory");
		this.connection = factory.createConnection();
		this.session = this.connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
		Destination destOut = (Destination) ctx.lookup(sendDest);
		Destination destIn = (Destination) ctx.lookup(receiveDest);
		this.producer = this.session.createProducer(destOut);
		this.consumer = this.session.createConsumer(destIn);
		this.consumer.setMessageListener(this);
		this.connection.start();
	}

	/**
	 * asynchronous message consumption
	 *
	 * @see javax.jms.MessageListener
	 */
	@Override
	public void onMessage(Message message) {
		if (message instanceof TextMessage) {
			TextMessage textMessage = (TextMessage) message;
			try {
				System.out.println(textMessage.getStringProperty("user") + "> " + textMessage.getText());
			} catch (JMSException e) {
				System.err.println(e);
			}
		}
	}

	/**
	 * main routine and starting point of program
	 *
	 * @param args[0] user name
	 */
	public static void main(String[] args) {
		ChatClientUserProperty node = null;
		try {
			node = new ChatClientUserProperty("var.mom.jms.channel1", "var.mom.jms.channel1");
			BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
			String line;
			while (true) {
				line = input.readLine();
				TextMessage msg = node.session.createTextMessage(line);
				msg.setStringProperty("user", args[0]);
				node.producer.send(msg);
			}
		} catch (NamingException | JMSException | IOException e) {
			System.err.println(e);
		} finally {
			try {
				if ((node != null) && (node.producer != null)) {
					node.producer.close();
				}
				if ((node != null) && (node.consumer != null)) {
					node.consumer.close();
				}
				if ((node != null) && (node.session != null)) {
					node.session.close();
				}
				if ((node != null) && (node.connection != null)) {
					node.connection.close();
				}
			} catch (JMSException e) {
				System.err.println(e);
			}
		}
	}
}